/*
 * OpenGL.h
 *
 * OpenGL header items for TRueView
 */
#ifndef OPENGL_H
#define OPENGL_H 1

#ifndef GL_EXTERN
#define GL_EXTERN extern
#endif // GL_EXTERN

#ifdef OPENGL_FULL   // don't include things that require GL.H et al unless we're in opengl.c
/*
 * Macros/Constants
 */
#define NORMAL_SCALE          16384.0  // divide normals by this to create unit normals
#define MAX_NUM_MESHES        768      // arbitrary number - really should malloc()
#define MAX_NUM_ROOM_MESHES   250      // arbitrary number - really should malloc() [Max in real TR2 is 193]
#define ESCAPE                0x1b     // the standard ASCII ESC key

// viewpoint perspective stuff
#define FOV_Y                 40.0     // field of view (degrees) in Y direction
#define CLIP_NEAR             0.1      // near clipping plane
#define CLIP_FAR              6000.0   // was 5000.0   // far clipping plane
#define TR2_COLOUR_MAX        64.0
#define TR2_COLOUR16_MAX      32.0

// constants for GLUT menu items
enum {
   GLUT_MENU_TOGGLE_ALPHA = 1,
   GLUT_MENU_TOGGLE_FOG,
   GLUT_MENU_TOGGLE_ITEM_MESHES,
   GLUT_MENU_TOGGLE_FACE_CULLING,
   GLUT_MENU_TOGGLE_LIGHTING,
   GLUT_MENU_TOGGLE_PORTALS,
   GLUT_MENU_RESET_CAMERA,
   GLUT_MENU_TOGGLE_ANIMATED_MESHES,
   GLUT_MENU_TOGGLE_SINGLE_ROOM_DISPLAY,
   GLUT_MENU_NEXT_SINGLE_ROOM,
   GLUT_MENU_PREVIOUS_SINGLE_ROOM,
   GLUT_MENU_TOGGLE_ANIMATED_TEXTURES,
   GLUT_MENU_TOGGLE_BOXES_DISPLAY,
   GLUT_MENU_TOGGLE_SKYMESH_DISPLAY,
   GLUT_MENU_SAVE_LEVEL,
   GLUT_MENU_DUMP_LEVEL,
   GLUT_MENU_TOGGLE_STATIC_MESHES,
   GLUT_MENU_TOGGLE_ROOM_SPRITES,
   GLUT_MENU_TOGGLE_VERTEX_LIGHTING,
   GLUT_MENU_RESET_WINDOW_SIZE,
   GLUT_MENU_EXIT = 99
   };

/*
 * Data Structures
 */
typedef struct {  // GLfloat version of tr2_vector
   GLfloat x;
   GLfloat y;
   GLfloat z;
   } opengl_vector;

typedef struct {  // GLfloat version of tr2_colour
   GLfloat r;
   GLfloat g;
   GLfloat b;
   GLfloat a;
   } opengl_colour;

typedef struct {
   GLfloat s;
   GLfloat t;
   } opengl_texture_coord;

typedef struct {
   int         w,
               h;        
   GLubyte     *data;
   GLuint      TextureID;
   texture     *tex;
   } opengl_texture;

typedef enum opengl_mesh_type {
   NONE,
   ITEM_MESH,
   ROOM_MESH
   } opengl_mesh_type;

typedef struct opengl_mesh_struct {
   opengl_mesh_type     type;
   opengl_vector        BoundingBoxLow,
                        BoundingBoxHigh;
   int                  AdjacentRooms[MAX_NUM_ROOM_MESHES], // size should be dynamic
                        NumAdjacentRooms;
   opengl_vector        Position;

   opengl_colour        *oglVertexColours;

   int                  oglNumTexturedRectangles;
   opengl_colour        *oglRectangleVertexColours;
   opengl_vector        *oglTexturedRectangleVertices;
   opengl_vector        *oglTexturedRectangleNormals;
   opengl_texture_coord *oglRectangleTextureCoordinates;
   opengl_texture       **oglRectangleTextures;

   int                  oglNumTexturedTriangles;
   opengl_colour        *oglTriangleVertexColours;
   opengl_vector        *oglTexturedTriangleVertices;
   opengl_vector        *oglTexturedTriangleNormals;
   opengl_texture_coord *oglTriangleTextureCoordinates;
   opengl_texture       **oglTriangleTextures;

   int                  oglNumColouredRectangles;
   opengl_colour        *oglCRectangleVertexColours;
   opengl_vector        *oglColouredRectangleVertices;
   opengl_vector        *oglColouredRectangleNormals;
   opengl_colour        *oglRectangleColours;

   int                  oglNumColouredTriangles;
   opengl_colour        *oglCTriangleVertexColours;
   opengl_vector        *oglColouredTriangleVertices;
   opengl_vector        *oglColouredTriangleNormals;
   opengl_colour        *oglTriangleColours;
   } opengl_mesh;

struct opengl_item_struct {
   opengl_mesh *RoomMeshes[MAX_NUM_ROOM_MESHES],   // size should be dynamic
               *ItemMeshes[MAX_NUM_MESHES];        // size should be dynamic
   opengl_texture **SpriteTextures;
   };

#endif   // OPENGL_FULL

/*
 * Function prototypes
 */
void opengl_init(int *argc, char **argv);
void opengl_main(TR2_Level *level);
#ifdef OPENGL_FULL   // don't include things that require GL.H et al unless we're in opengl.c
void opengl_resize(int width, int height);
void opengl_draw(void);
void opengl_draw_room_mesh(opengl_mesh *mesh);
void opengl_draw_item_mesh(opengl_mesh *mesh);
void opengl_key_special(int key, int x, int y);
void opengl_key(unsigned char key, int x, int y);
void opengl_free_item(void);
void opengl_free_item_mesh(opengl_mesh *mesh);
void opengl_set_room_mesh(int which);
void opengl_set_item_mesh(int which);
void opengl_print(unsigned char *string);
opengl_mesh *opengl_create_room_mesh(int which);
opengl_mesh *opengl_create_item_mesh(int which);
opengl_texture *opengl_add_texture(TextureType type, int TextureNumber, TexturePolygonType polygon_type);
int WhichRoom(GLfloat x, GLfloat y, GLfloat z);
int WhichRoomCamera(void);
extern void *TR2alloc(int size);
void opengl_free_all_meshes(void);
void opengl_create_all_meshes(void);
void opengl_free_mesh(opengl_mesh **mesh);
void opengl_init_sprite_textures(void);

/*
 * Data
 */
GL_EXTERN GLuint    *TexTileIDs;
GL_EXTERN TR2_Level *opengl_level;
GL_EXTERN int       opengl_mipmap_textures;
GL_EXTERN int       OpenGL_TextureListOffset;
GL_EXTERN opengl_texture *OpenGL_TextureList;
GL_EXTERN struct opengl_item_struct opengl_item;
GL_EXTERN int       FastOpenGL;

#endif   // OPENGL_FULL
#endif   // ifdef OPENGL_H
